part of TeamSpeak;

class Channel extends TeamSpeakEntity {
  final int id;
  int _parentId;
  int get parentId => _parentId;
  bool get hasParent => _parentId > 0;
  
  String get name => _name; String _name;
  String get topic => _topic; String _topic;
  String get description => _description; String _description;
  
  Channel _parentChannel; bool _parentChannelSet = false;
  Channel get parentChannel => _parentChannel;
  List<Channel> get children => server.channels.values
      .where((Channel c) => c.parentChannel == this).toList(growable: false);
  List<User> get users => server.users.values
      .where((User u) => u.channel == this).toList(growable: false);
  
  /// Moves the channel so it will be a child of Channel p
  Future<Channel> setParentChannel(Channel p) {
    // If no parent has ever been set, this is considered as an initialization so 
    // we don't have to notify the server.
    if(!_parentChannelSet){
      if(p.id != parentId){
        throw new StateError("Can't mark channel with id ${p.id} as parent since parent should have id ${parentId}!");
      }
      _parentChannel = p;
      _parentChannelSet = true;
    } else {
      // Move the channel so it will be a child of channel p
      return _client.send(new Command("channelmove", parameters: {"cid": id, "cpid": p.id}))
        .then((Answer answer){
          _parentId = p.id;
          _parentChannel = p;
          return this;
        });
    }
  }
  
  Channel(Server server, _Client _client, int this.id, int this._parentId) 
      : super(server, _client);
  
  Future<Channel> update(){
    return _client.send(new Command("channelinfo", parameters: {"cid": id}))
        .then((Answer answer){
          final Map<String, String> info = answer.items[0];
          _name = info["channel_name"];
          _topic = info["channel_topic"];
          _description = info["channel_description"];
          return this;
        });
  }
}